/* --------------------------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var fs = require("fs");
var LocalizeInfo;
(function (LocalizeInfo) {
    function is(value) {
        var candidate = value;
        return candidate && isDefined(candidate.key) && isDefined(candidate.comment);
    }
    LocalizeInfo.is = is;
})(LocalizeInfo || (LocalizeInfo = {}));
var ExtensionKind;
(function (ExtensionKind) {
    ExtensionKind[ExtensionKind["prePackaged"] = 1] = "prePackaged";
    ExtensionKind[ExtensionKind["marketPlace"] = 2] = "marketPlace";
})(ExtensionKind = exports.ExtensionKind || (exports.ExtensionKind = {}));
var _options;
var _isPseudo;
var _extensionKind;
var _root;
var _outDir;
var _outPath;
var _resolvedLanguage;
// If undefined we never tried to load. If null we tried to load (the bundle exists on disk)
// but the actual load failed.
var _resolvedBundle;
var _resolvedCacheLocation;
function initializeSettings() {
    _options = { locale: undefined, cacheLanguageResolution: true };
    _isPseudo = false;
    _extensionKind = undefined;
    _root = undefined;
    _outDir = undefined;
    _outPath = undefined;
    _resolvedLanguage = undefined;
    _resolvedBundle = undefined;
    _resolvedCacheLocation = undefined;
}
initializeSettings();
var toString = Object.prototype.toString;
function isDefined(value) {
    return typeof value !== 'undefined';
}
function isNumber(value) {
    return toString.call(value) === '[object Number]';
}
function isString(value) {
    return toString.call(value) === '[object String]';
}
function isBoolean(value) {
    return value === true || value === false;
}
function format(message, args) {
    var result;
    if (_isPseudo) {
        // FF3B and FF3D is the Unicode zenkaku representation for [ and ]
        message = '\uFF3B' + message.replace(/[aouei]/g, '$&$&') + '\uFF3D';
    }
    if (args.length === 0) {
        result = message;
    }
    else {
        result = message.replace(/\{(\d+)\}/g, function (match, rest) {
            var index = rest[0];
            return typeof args[index] !== 'undefined' ? args[index] : match;
        });
    }
    return result;
}
function createScopedLocalizeFunction(messages) {
    return function (key, message) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        if (isNumber(key)) {
            if (key >= messages.length) {
                console.error("Broken localize call found. Index out of bounds. Stacktrace is\n: " + new Error('').stack);
                return;
            }
            return format(messages[key], args);
        }
        else {
            if (isString(message)) {
                console.warn("Message " + message + " didn't get externalized correctly.");
                return format(message, args);
            }
            else {
                console.error("Broken localize call found. Stacktrace is\n: " + new Error('').stack);
            }
        }
    };
}
function localize(key, message) {
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    return format(message, args);
}
function tryResolveBundle(root) {
    var locale = _options.locale;
    while (locale) {
        var candidate = path.join(root, "nls.bundle." + locale + ".json");
        if (fs.existsSync(candidate)) {
            return candidate;
        }
        else {
            var index = locale.lastIndexOf('-');
            if (index > 0) {
                locale = locale.substring(0, index);
            }
            else {
                locale = undefined;
            }
        }
    }
    // Test if we can reslove the default bundle.
    if (locale === undefined) {
        var candidate = path.join(root, 'nls.bundle.json');
        if (fs.existsSync(candidate)) {
            return candidate;
        }
    }
    return undefined;
}
function resolveLanguage(file) {
    var resolvedLanguage;
    if (_options.cacheLanguageResolution && _resolvedLanguage) {
        resolvedLanguage = _resolvedLanguage;
    }
    else {
        if (_isPseudo || !_options.locale) {
            resolvedLanguage = '.nls.json';
        }
        else {
            var locale = _options.locale;
            while (locale) {
                var candidate = '.nls.' + locale + '.json';
                if (fs.existsSync(file + candidate)) {
                    resolvedLanguage = candidate;
                    break;
                }
                else {
                    var index = locale.lastIndexOf('-');
                    if (index > 0) {
                        locale = locale.substring(0, index);
                    }
                    else {
                        resolvedLanguage = '.nls.json';
                        locale = null;
                    }
                }
            }
        }
        if (_options.cacheLanguageResolution) {
            _resolvedLanguage = resolvedLanguage;
        }
    }
    return file + resolvedLanguage;
}
function mkdir(directory) {
    try {
        fs.mkdirSync(directory);
    }
    catch (err) {
        if (err.code === 'EEXIST') {
            return;
        }
        else if (err.code === 'ENOENT') {
            var parent = path.dirname(directory);
            if (parent !== directory) {
                mkdir(parent);
                fs.mkdir(directory);
            }
        }
        else {
            throw err;
        }
    }
}
function supportsLanguagePack() {
    if (!_root || !_outDir || !_options._languagePackLocation
        || (_extensionKind === ExtensionKind.prePackaged && !_options._resolvedLanguagePackExtensionLocation)
        || (_extensionKind === ExtensionKind.marketPlace && !_options._cacheRoot)) {
        return false;
    }
    return fs.existsSync(path.join(_outPath, 'nls.metadata.json'));
}
function loadDefaultBundle() {
    var metaData = require(path.join(_outPath, 'nls.metadata.json'));
    var result = Object.create(null);
    for (var module_1 in metaData.content) {
        var entry = metaData.content[module_1];
        result[module_1] = entry.messages;
    }
    return result;
}
function createLanguageBundle(extensionName) {
    var languagePack = require(path.join(_options._languagePackLocation, 'translations', 'extensions', extensionName + ".i18n.json"));
    var metaData = require(path.join(_outPath, 'nls.metadata.json'));
    var result = Object.create(null);
    for (var module_2 in metaData.content) {
        var entry = metaData.content[module_2];
        var translations = languagePack[_outDir + "/" + module_2];
        if (translations) {
            var resultMessages = [];
            for (var i = 0; i < entry.keys.length; i++) {
                var messageKey = entry.keys[i];
                var key = isString(messageKey) ? messageKey : messageKey.key;
                var translatedMessage = translations[key];
                if (translatedMessage === undefined) {
                    translatedMessage = entry.messages[i];
                }
                resultMessages.push(translatedMessage);
            }
            result[module_2] = resultMessages;
        }
        else {
            result[module_2] = entry.messages;
        }
    }
    return result;
}
function loadPrepackagedLanguagePackBundle() {
    var extensionName = require(path.join(_root, 'package.json')).name;
    var root = path.join(_options._resolvedLanguagePackExtensionLocation);
    var bundle = path.join(root, extensionName + ".nls.json");
    var useMemoryOnly = false;
    var noEntry = false;
    try {
        return JSON.parse(fs.readFileSync(bundle, { encoding: 'utf8', flag: 'r' }));
    }
    catch (err) {
        if (err.code === 'ENOENT') {
            noEntry = true;
        }
        else if (err instanceof SyntaxError) {
            // We have a syntax error. So no valid JSON. Use
            console.error("Syntax error parsing message bundle: " + err.message);
            useMemoryOnly = true;
        }
        else {
            throw err;
        }
    }
    var result = createLanguageBundle(extensionName);
    if (useMemoryOnly) {
        return result;
    }
    if (noEntry) {
        mkdir(root);
        try {
            fs.writeFileSync(bundle, JSON.stringify(result), { encoding: 'utf8', flag: 'wx' });
        }
        catch (err) {
            if (err.code === 'EEXIST') {
                return result;
            }
            throw err;
        }
    }
    return result;
}
function loadMarketPlaceLanguagePackBundle() {
    var packageJson = require(path.join(_root, 'package.json'));
    var extensionName = packageJson.name;
    var extensionVersion = packageJson.version;
    var extensionPublisher = packageJson.publisher;
    if (!extensionName || !extensionVersion || !extensionPublisher) {
        return undefined;
    }
    var root = path.join(_options._cacheRoot, extensionName + "." + extensionPublisher + "-" + extensionVersion);
    var useMemoryOnly = false;
    var noEntry = false;
    // We already have a cached bundle. Use it.
    var bundle = tryResolveBundle(root);
    var result;
    if (bundle) {
        try {
            result = JSON.parse(fs.readFileSync(bundle, { encoding: 'utf8', flag: 'r' }));
            try {
                // Touch last used.
                fs.writeFileSync(path.join(root, 'lastUsed.touch'), '', { encoding: 'utf8', flag: 'w' });
            }
            catch (err) {
                // Ignore error.
            }
            return result;
        }
        catch (err) {
            if (err.code === 'ENOENT') {
                noEntry = true;
            }
            else if (err instanceof SyntaxError) {
                // We have a syntax error. So no valid JSON. Use
                console.error("Syntax error parsing message bundle.", err);
                useMemoryOnly = true;
            }
            else {
                throw err;
            }
        }
    }
    result = createLanguageBundle(extensionName);
    if (useMemoryOnly) {
        return result;
    }
    if (noEntry) {
        mkdir(root);
        try {
            fs.writeFileSync(bundle, JSON.stringify(result), { encoding: 'utf8', flag: 'wx' });
        }
        catch (err) {
            if (err.code === 'EEXIST') {
                return result;
            }
            throw err;
        }
    }
    return result;
}
function loadMessageBundle(file) {
    if (!file) {
        // No file. We are in dev mode. Return the default
        // localize function.
        return localize;
    }
    // Remove extension since we load json files.
    var ext = path.extname(file);
    if (ext) {
        file = file.substr(0, file.length - ext.length);
    }
    // We have a single resolved bundle.
    if (_resolvedBundle !== undefined) {
        // We failed resolve the bundle including generating one from meta data.
        // We return a special localize function that always errors
        if (_resolvedBundle === null) {
            return function () {
                return 'Failed to load message bundle. See console for details.';
            };
        }
        if (!file.startsWith(_outPath)) {
            console.error("Mismatch between out path(" + _outPath + ") and file location(" + file + ")");
            return function () {
                return 'Location mismatch. See console for details.';
            };
        }
        var module_3 = file.substr(_outPath.length + 1).replace(/\\/g, '/');
        var messages = _resolvedBundle[module_3];
        if (messages === undefined) {
            console.error("Messages for file " + file + " not found. See console for details.");
            return function () {
                return 'Messages not found';
            };
        }
        return createScopedLocalizeFunction(messages);
    }
    // Try to load a single file bundle
    try {
        var json = require(resolveLanguage(file));
        if (Array.isArray(json)) {
            return createScopedLocalizeFunction(json);
        }
        else {
            if (isDefined(json.messages) && isDefined(json.keys)) {
                return createScopedLocalizeFunction(json.messages);
            }
            else {
                console.error("String bundle '" + file + "' uses an unsupported format.");
                return function () {
                    return 'File bundle has unsupported format. See console for details';
                };
            }
        }
    }
    catch (err) {
        console.error("Can't load string bundle for " + file, err);
        return function () {
            return 'Failed to load file bundle. See console for details.';
        };
    }
}
exports.loadMessageBundle = loadMessageBundle;
function config(opt, rootOrKind, outDirOrRoot, outDir) {
    initializeSettings();
    var options;
    if (isString(opt)) {
        try {
            options = JSON.parse(opt);
        }
        catch (e) {
            console.error("Error parsing nls options: " + opt);
        }
    }
    else {
        options = opt;
    }
    var kind;
    var root;
    if (isString(rootOrKind)) {
        kind = undefined;
        root = rootOrKind;
        outDir = outDirOrRoot;
    }
    else {
        kind = rootOrKind;
        root = outDirOrRoot;
    }
    if (options) {
        if (isString(options.locale)) {
            _options.locale = options.locale.toLowerCase();
            _resolvedLanguage = undefined;
            _resolvedCacheLocation = undefined;
        }
        var asInternal = options;
        if (isString(asInternal._cacheRoot)) {
            _options._cacheRoot = asInternal._cacheRoot;
        }
        if (isString(asInternal._languagePackLocation)) {
            _options._languagePackLocation = asInternal._languagePackLocation;
        }
        if (isString(asInternal._resolvedLanguagePackCoreLocation)) {
            _options._resolvedLanguagePackCoreLocation = asInternal._resolvedLanguagePackCoreLocation;
        }
        if (isString(asInternal._resolvedLanguagePackExtensionLocation)) {
            _options._resolvedLanguagePackExtensionLocation = asInternal._resolvedLanguagePackExtensionLocation;
        }
        if (isBoolean(options.cacheLanguageResolution)) {
            _options.cacheLanguageResolution = options.cacheLanguageResolution;
        }
    }
    var packageJsonExists = false;
    if (kind === ExtensionKind.prePackaged && isString(root) && root.length > 0 && !outDir) {
        var parent = path.dirname(root);
        var basename = path.basename(root);
        if (fs.existsSync(path.join(parent, 'package.json'))) {
            packageJsonExists = true;
            root = parent;
            outDir = basename;
        }
    }
    if (root && path.isAbsolute(root)) {
        _root = root;
        _outDir = outDir;
        if (_outDir) {
            _outPath = path.join(_root, _outDir);
        }
    }
    if (kind && _root && _outDir && (packageJsonExists || fs.existsSync(path.join(root, 'package.json')))) {
        _extensionKind = kind;
    }
    _isPseudo = _options.locale === 'pseudo';
    // We have a root and a outDir. So we can try to load a bundle if it exists.
    if (_root && _outDir) {
        // We are using a language pack.
        if (supportsLanguagePack()) {
            try {
                if (kind === ExtensionKind.prePackaged) {
                    _resolvedBundle = loadPrepackagedLanguagePackBundle();
                }
                else {
                    _resolvedBundle = loadMarketPlaceLanguagePackBundle();
                }
                return loadMessageBundle;
            }
            catch (err) {
                console.error("Loading the message bundle from language pack failed with exception", err);
            }
        }
        var candidate = tryResolveBundle(path.join(_root, _outDir));
        if (candidate) {
            try {
                _resolvedBundle = require(candidate);
                return loadMessageBundle;
            }
            catch (err) {
                console.error("Loading in the box message bundle failed.", err);
            }
        }
        try {
            _resolvedBundle = loadDefaultBundle();
            return loadMessageBundle;
        }
        catch (err) {
            console.error("Generating default bundle from meta data failed.", err);
            _resolvedBundle = null;
        }
    }
    return loadMessageBundle;
}
exports.config = config;
//# sourceMappingURL=main.js.map